""" Install.py of PVSPloneGeneral : usefull only for installing Workflow."""

import traceback
from cStringIO import StringIO
from Acquisition import aq_base
from Products.CMFCore.utils import getToolByName
from Products.Archetypes.public import listTypes
from Products.Archetypes.Extensions.utils import installTypes, install_subskin
from Products.CMFCore.DirectoryView import addDirectoryViews
from Products.PVSForPlone2 import install_globals
from Products.CMFCore.CMFCorePermissions import ModifyPortalContent, View, \
     AccessContentsInformation

from Products.PVSForPlone2.PVSPermissions import *
from Products.PVSForPlone2.PVSTool import manage_addPVSTool


def install_pvs_workflow(self, out):
    workflow_tool = getToolByName(self, 'portal_workflow')

    if 'pvs_workflow' in workflow_tool.objectIds():
        workflow_tool._delObject('pvs_workflow')

    old_pwf = workflow_tool._getOb('plone_workflow')
    pwf = workflow_tool.manage_clone(old_pwf, 'pvs_workflow')

    # add new state 'archived'
    if not hasattr(pwf.states, 'archived'):
        pwf.states.addState('archived')
        sdef = pwf.states.archived
        sdef.setProperties( transitions=('publish',) )
        sdef.setPermission(View, 0, ('Manager', 'Owner'))
        sdef.setPermission(AccessContentsInformation, 0, \
                           ('Manager', 'Owner'))
        # no one is allowed to modify objects in archived state
        sdef.setPermission(ModifyPortalContent, 0, () )

    # add new transition 'archive'
    # it's invisible to user action
    if not hasattr(pwf.transitions, 'archive'):
        pwf.transitions.addTransition('archive')
        tdef = pwf.transitions.archive
        tdef.setProperties( title='', new_state_id='archived' )

    # published objects can transit to the 'archived' state
    sdef = pwf.states.published
    if 'archive' not in sdef.transitions:
        sdef.setProperties( transitions=tuple(sdef.transitions)+('archive',) )

    workflow_tool.setDefaultChain('pvs_workflow')
    print >> out, "Successfully installed pvs_workflow."


def install(self):
    out = StringIO()
    # instantiate the versioning tool
    portal = getToolByName(self, 'portal_url').getPortalObject()

    portal.manage_addProduct['PVSForPlone2'].manage_addPVSTool()

    action_tool = getToolByName(self, 'portal_actions')
    action_tool.addActionProvider('portal_content_versioning_system')

    #Join PVS skin folder to the skin selection
    skinstool = getToolByName(self, 'portal_skins')

    if 'PVS' not in skinstool.objectIds():
        addDirectoryViews(skinstool, 'skins', install_globals)

    skins = skinstool.getSkinSelections()
    for skin in skins:
        path = skinstool.getSkinPath(skin)
        # insert before content skin
        if path.find('PVS') < 0:
            path = path.split('custom,')
            path = path[0] + 'custom,PVS,' + path[1]
            skinstool.addSkinSelection(skin, path)

    catalog_tool = getToolByName(self, 'portal_catalog')
    if not catalog_tool._catalog.indexes.has_key('version_cluster_id'):
        catalog_tool.manage_addIndex('version_cluster_id', 'FieldIndex')

    install_pvs_workflow(self, out)

    print >> out, "Successfully installed PVS for Plone 2."

    pvs_configlet = {
       'id': 'pvs_parameters',
       'appId': 'PVS',
       'name': 'PVS parameters',
       'action': 'string:$portal_url/pvs_config_form',
       'category': 'Products',
       'permission': ('Manage portal',),
#       'imageUrl': 'pvs_icon.png',
       }


    self.portal_controlpanel.registerConfiglet(**pvs_configlet)
    return out.getvalue()


def uninstall(self):
    out = StringIO()
    action_tool = getToolByName(self, 'portal_actions')
    action_tool.deleteActionProvider('portal_content_versioning_system')

    # remove the skin from the skin selection
    skinstool = getToolByName(self, 'portal_skins')
    if 'PVS' in skinstool.objectIds():
        skinstool.manage_delObjects('PVS')
    skins = skinstool.getSkinSelections()
    for skin in skins:
        path = skinstool.getSkinPath(skin)
        if path.find('PVS') >= 0:
            path = path.replace('PVS,', '')
            skinstool.addSkinSelection(skin, path)

    # remove relevant workflows
    wftool = getToolByName(self, 'portal_workflow')
    wftool.manage_delObjects(ids=['pvs_workflow'])
    wftool.setDefaultChain('plone_workflow')
    # remove the tool instance
    portal = getToolByName(self, 'portal_url').getPortalObject()
    portal.manage_delObjects('portal_content_versioning_system')
    print >> out, "pvs_workflow removed. Restoring plone_workflow as default."

    self.portal_controlpanel.unregisterConfiglet('pvs_parameters')

    print >> out, 'PVS for Plone 2 sucessfully uninstalled.'

    return out.getvalue()


