import sys
from time import time

from OFS.SimpleItem import SimpleItem
from Globals import InitializeClass, PersistentMapping, DTMLFile
from AccessControl import ClassSecurityInfo
from AccessControl.Permissions import delete_objects
from AccessControl import getSecurityManager
from Acquisition import aq_base, aq_inner, aq_parent, aq_chain

from Products.CMFCore.CMFCorePermissions import ManagePortal, AccessContentsInformation, ModifyPortalContent, ReviewPortalContent, View, ManageProperties, RequestReview
from Products.CMFCore.utils import UniqueObject
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.ActionProviderBase import ActionProviderBase
from Products.CMFCore.ActionInformation import ActionInformation
from Products.CMFCore.Expression import Expression
from Products.CMFCore.DirectoryView import addDirectoryViews
from Products.CMFCore.WorkflowCore import WorkflowException
from Products.DCWorkflow.utils import modifyRolesForPermission

import re

from PVSPermissions import CreateVersions
from urllib import urlencode as encode

from config import *

_marker = []  # Create a new marker object.

def manage_addPVSTool(self, REQUEST=None):
    " "
    # raise err if not in portal root
    if hasattr(self, 'portal_url'):
        obj = PVSTool()
        if not hasattr(self, obj.getId()):
            self._setObject(obj.getId(), obj)
        if REQUEST: return self.manage_main(self, REQUEST)
    else:
        raise 'Wrong place', 'The current folder is not a CMF Site. PVSForPlone2 tool can only be added to the root of a CMF Site.'



class PVSTool(UniqueObject, SimpleItem, ActionProviderBase):

    """ Provide versioning mechanism to CMF
    """
    id = 'portal_content_versioning_system'
    meta_type = 'Portal content Versioning System'

    unlimited_version_creation = False

    manage_options = ({ 'label' : 'Overview', 'action' : 'manage_overview' }, ) + SimpleItem.manage_options

    manage_overview = DTMLFile('dtml/explainPVS', globals())

    _actions = (ActionInformation(id='versionize'
                                , title='Versionize'
                                , action=Expression(
                                    text="string: ${object_url}/versionize_form"
                                        )
                                , permissions=(CreateVersions, )
                                , category='object'
                                , visible=1
                                 )
                ,ActionInformation(id='manage_versions'
                                , title='All Versions'
                                , action=Expression(
                                    text='string: ${object_url}/manage_versions')
                                , condition=Expression(
                                    text="python: len(object.portal_content_versioning_system.getAllVersionIds(object)) > 1")
                                , permissions=(CreateVersions, ReviewPortalContent)
                                , category='object'
                                , visible=1
                                 )
               )

    security = ClassSecurityInfo()

    # ----------- util methods for object verioning -------------
    #
    def allowUnlimitedVersionCreation(self):
        # return True either if the system is set so or user has review permission
        user = getSecurityManager().getUser()
        context = self.aq_parent
        return self.unlimited_version_creation or user.has_permission(ReviewPortalContent, context)

    security.declareProtected(ManagePortal, 'editProperties')
    def editProperties(self, REQUEST):
        v = REQUEST.get('unlimited_version_creation', None)
        if v:
            self.unlimited_version_creation = v

    def getAllVersionIds(self, obj):
        pcat = getToolByName(self, 'portal_catalog')
        rslt = obj.version_cluster_id > 0 and pcat(version_cluster_id = obj.version_cluster_id) or []
        return [x.getId for x in rslt]

    def getVersionCluster(self, obj):
        ids = self.getAllVersionIds(obj)
        return [getattr(obj.aq_parent, id) for id in ids if hasattr(obj.aq_parent, id)]

    def getVersionClusterForUser(self, obj):
        user = getSecurityManager().getUser()
        return [x for x in self.getVersionCluster(obj) if user.has_permission(View, x)]

    def getUnapprovedVersions(self):
        """ Return versions:
            that is not published or archived
            and user has permission to retract it.
        """
        context = self.aq_parent
        versions = self.getVersionCluster(context)
        return [v for v in versions if self.unapproved(v)]

    def unapproved(self, obj):

        workflow_tool = getToolByName(self, 'portal_workflow')
        not_published = workflow_tool.getInfoFor(obj, 'review_state', '') not in ('published', 'archived')

        user = getSecurityManager().getUser()
        user_can_retract = user.has_permission(RequestReview, obj)

        return not_published and user_can_retract


    def _versionHasChildren(self, id, vids):
        " "
        prefix = id + '.'
        return [id for id in vids if id.startswith(prefix)]

    def getTopVersionId(self, obj):
        version_id = obj.getId()
        if version_id.find('~v~') != -1:
            return version_id[:version_id.find('~v~')]
        else:
            return version_id

    def delVersions(self, obj, ids):
        " "
        for id in ids:
            if id.find('~v~') < 0:
                raise 'Delete Error', '%s is a frontend object, therefore cannot be deleted from version management' % id
        container = aq_parent(aq_inner(obj))
        user = getSecurityManager().getUser()
        for ob in [container._getOb(id) for id in ids]:
            if not user.has_permission(ModifyPortalContent, ob):
                raise 'Unauthorized'
        for id in ids:
            container._delObject(id)

    def createVersion(self):
        """ Create a new Version
        """
        orig_obj = self.aq_parent

        user = getSecurityManager().getUser()
        if not user.has_permission(CreateVersions, orig_obj):
            raise 'Unauthorized'

        container = aq_parent(aq_inner(orig_obj))
        if getattr(aq_base(orig_obj), 'version_cluster_id', -1) == -1:
            # this object isn't verionized. shape it up
            orig_obj.version_cluster_id = int(time())
            orig_obj.version_number = '1'

        new_id = getIdForNewVersion(orig_obj)
        new_version = orig_obj._getCopy(container)
        # check if folderish
        if hasattr(aq_base(orig_obj), 'objectIds'):
            # strip off all sub-objects
            for id in new_version.objectIds():
                new_version._delOb(id)
            # reset the _objects index
            new_version._objects = ()

            # disable folder_contents view from all users (including manager)
            # dispite the setting of the workflow
            # wrap new_context with acquisition first
            nv = new_version.__of__(orig_obj)
            modifyRolesForPermission(nv, 'List folder contents', ())

        # nobody is allowed to use delete_object on hidden version, use delVersion istead
        modifyRolesForPermission(nv, 'Delete objects', ())

        new_version._setId(new_id)
        new_version.version_number = new_id.split('~v~')[-1]
        container._setOb(new_id, new_version)
        new_context = container._getOb(new_id)

        # plone and azcat formaker will rename the obj if not set
        self.REQUEST.set('id', new_id)

        # set ownership
        new_context.manage_fixupOwnershipAfterAdd()
        user = getSecurityManager().getUser()
        if hasattr(new_context, '__ac_local_roles__') and user is not None:
            name=user.getUserName()
            if name != 'Anonymous User':
                new_context.__ac_local_roles__ = {name : ['Owner']}

        new_context.notifyWorkflowCreated() # set new version into the default state
        orig_obj.reindexObject() # expose the object as versionized to the catalog
        new_context.reindexObject()
        msg = '?portal_status_message=A+new+version+has+been+created.'
        self.REQUEST.RESPONSE.redirect(new_context.absolute_url()+msg)

    def switchPublished(self, to_be_published):
        "Switch the published obj with the archived one"
        user = getSecurityManager().getUser()
        if not user.has_permission(RequestReview, to_be_published):
            raise 'Unauthorized'

        container = aq_parent(aq_inner(to_be_published))
        # remove the obj from the ZODB tree
        container._delObject(to_be_published.getId())

        front_end_id = to_be_published.getId().split('~v~')[0]
        to_be_archived = aq_base(container._getOb(front_end_id))
        to_be_archived_id = front_end_id + '~v~' + to_be_archived.version_number

        # switch ids
        to_be_archived._setId(to_be_archived_id)
        to_be_published._setId(front_end_id)

        # if folderish
        if getattr(aq_base(to_be_published), 'isAnObjectManager', 0):
            for name, subob in to_be_archived.objectItems():
                to_be_published._setOb(name, aq_base(subob))
                to_be_archived._delOb(name)
            to_be_published._objects = to_be_archived._objects
            to_be_archived._objects = ()
            # passing [] as roles, so that who having the permission is acquired from above
            modifyRolesForPermission(to_be_published, 'List folder contents', [])

        # restore the defult permission setting for delete object
        modifyRolesForPermission(to_be_published, 'Delete objects', [])

        # switch objs
        container._setOb(to_be_archived.getId(), to_be_archived)
        container._setOb(front_end_id, aq_base(to_be_published))

        the_archived = container._getOb(to_be_archived_id)
        new_context = container._getOb(front_end_id)

        workflow_tool = getToolByName(self, 'portal_workflow')
        # check if the archived item was published
        review_state = workflow_tool.getInfoFor(the_archived, 'review_state')
        if review_state == 'published':
            # try perform 'archive' transition
            try:
                workflow_tool.doActionFor(the_archived, 'archive')
            except WorkflowException:
                # the associated workflow is not versionized
                # set the archived into default state
                the_archived.notifyWorkflowCreated()

        # force to reindex everything on the two objects
        # the workflow methods, such as notifyWorkflowCreated, doActionFor etc.
        # will only reindex workflow related variables
        the_archived.reindexObject()
        new_context.reindexObject()

        # this seems to fix a strange ValueError: Cache values may only be in one cache
        get_transaction().commit()
        return new_context

    def ori_content_status_modify(self, new_context, workflow_action=None, comment='', effective_date=None, expiration_date=None, *args):
        skinstool = getToolByName(self, 'portal_skins')
        script = skinstool.plone_form_scripts.content_status_modify
        script = script.__of__(new_context)
        return  script(workflow_action, comment, effective_date, expiration_date, *args)

    def getURLtoCreateVersion(self, object):
        url = self.absolute_url() + '/createVersion?object_url='
        url += ('%2f'.join(object.getPhysicalPath())).replace('~', '%7e')
        return url

def getIdForNewVersion(obj):
    pvs = getToolByName(obj, 'portal_content_versioning_system')
    ids = pvs.getAllVersionIds(obj)
    prefix = obj.getId()

    t = prefix.split('~v~')
    if len(t) == 2:
        front_id, current_version_number = t
        # append the proper versionized id of the front end object to ids
        front_end_obj = obj.aq_parent._getOb(front_id)
        if hasattr(front_end_obj, 'version_number'):
            front_version_number = front_end_obj.version_number
            ids.append(front_id + '~v~' + front_version_number)
    else:
        front_id = t[0]
        current_version_number = getattr(obj, 'version_number', '1')
        prefix += '~v~' + current_version_number

    new_version_number = current_version_number.split('.')
    new_version_number[-1] = str(int(new_version_number[-1]) + 1)
    new_id = front_id + '~v~' + '.'.join(new_version_number)
    if new_id in ids:
        new_id = prefix + '.1'
        while new_id in ids:
            new_id += '.1'
        return new_id
    else:
        return new_id

    child_version_ids = [id for id in ids if re.match(prefix + '.\d+\Z', id)]
    id_suffixes = [int(id.split('.')[-1]) for id in child_version_ids]
    if id_suffixes:
        new_suffix = max(id_suffixes) + 1
    else:
        new_suffix = 1
    return '%s.%d' % (prefix, new_suffix)

InitializeClass(PVSTool)



